@@ -7,7 +7,9 @@ import jwt
 import requests
 from bs4 import BeautifulSoup
 
-from .utils import generate_otp
+from .exceptions import CertificateNotApprovedException
+
+from .utils import generate_otp, CertificateStatus, UserRole
 
 # Initialize logger
 logger = logging.getLogger(__name__)
@@ -25,12 +27,21 @@ class HaricaClient:
         email (str): The email of the user for login.
         password (str): The password of the user for login.
         totp_seed (str): Optional TOTP seed for 2FA.
+        http_proxy (str): Optional proxy configuration for HTTP requests.
+        https_proxy (str): Optional proxy configuration for HTTPS requests.
         environment (str): The environment of the CertManager ("dev", "stg", or "production").
         refresh_interval (int): Time in seconds before the JWT token needs to be refreshed.
     """
 
     def __init__(
-        self, email: str, password: str, totp_seed: str = None, environment: str = "production", refresh_interval: int = 3600
+        self,
+        email: str,
+        password: str,
+        totp_seed: str = None,
+        http_proxy: str = None,
+        https_proxy: str = None,
+        environment: str = "production",
+        refresh_interval: int = 3600,
     ):
         self.environment = environment
         self.email = email
@@ -40,8 +51,17 @@ class HaricaClient:
         self.login_lock = Lock()
         self.base_url = self.get_base_url()  # Set the base URL depending on the environment
         self.session = requests.Session()  # Reuse session for efficient requests
+
+        # Configure proxies if provided
+        if any([http_proxy, https_proxy]):
+            self.session.proxies.update({"http": http_proxy, "https": https_proxy})
+            logger.info(f"Configured proxies: {self.proxies}")
+
         self.token = None  # JWT token
         self.request_verification_token = None  # CSRF token
+        self.roles = set()
+        self.full_name = None
+
         self.prepare_client(force=False)  # Prepare client on initialization
 
     def prepare_client(self, force=False):
@@ -57,7 +77,12 @@ class HaricaClient:
             if self.token_is_valid() and not force:
                 return
 
-            self.login()  # Perform login
+            try:
+                self.login()
+            except requests.exceptions.ProxyError as ex:
+                logger.error(f"❌ Proxy connection error: {ex}")
+            except Exception as ex:
+                logger.error(f"❌ Login failed: {ex}")
 
     def token_is_valid(self):
         """
@@ -119,6 +144,19 @@ class HaricaClient:
         Logs in to CertManager and retrieves the necessary JWT token.
         If TOTP seed is provided, it uses 2FA during login.
         """
+
+        def parse_roles(raw_roles: str) -> set[UserRole]:
+            """Convert a comma-separated role string into a set of UserRole enums."""
+            roles = set()
+
+            for role in raw_roles.split(","):
+                role = role.strip()
+                # Validate if it's a valid enum value
+                if role in UserRole._value2member_map_:
+                    roles.add(UserRole(role))
+
+            return roles
+
         login_payload = {"email": self.email, "password": self.password}
 
         # If TOTP seed is provided, generate OTP and add it to the payload
@@ -126,11 +164,21 @@ class HaricaClient:
             login_payload.update({"token": generate_otp(self.totp_seed)})
             login_response = self.__make_post_request("/api/User/Login2FA", data=login_payload)
         else:
+            logger.debug("No TOTP seed provided. Using password-based login.")
             login_response = self.__make_post_request("/api/User/Login", data=login_payload)
 
+        login_response.raise_for_status()
+
         self.token = login_response.text
         if not self.token:
-            raise Exception("Failed to retrieve JWT token.")
+            if self.totp_seed:
+                raise Exception("Failed to retrieve JWT token.")
+            else:
+                raise Exception(
+                    "Failed to retrieve JWT token. You attempted to log in without "
+                    "2FA, but it may be enabled on your account. If it is, please "
+                    "provide the TOTP seed."
+                )
 
         # Update session headers with the JWT token for subsequent requests
         self.session.headers.update(
@@ -141,6 +189,10 @@ class HaricaClient:
 
         logger.debug("Login successful.")
 
+        current_logged_in_user = self.get_logged_in_user_profile()
+        self.roles = parse_roles(current_logged_in_user["role"])
+        self.full_name = current_logged_in_user["fullName"]
+
     def get_logged_in_user_profile(self):
         """
         Retrieves the profile of the logged-in user.
@@ -161,26 +213,140 @@ class HaricaClient:
         Returns:
             dict: The certificate details.
         """
-        # response_data = self.__make_post_request("/api/Certificate/GetCertificate", data={"id": certificate_id}).json()
-        response_data = self.__make_post_request(
-            "/api/OrganizationAdmin/GetEnterpriseCertificate", data={"id": certificate_id}
-        ).json()
-        return response_data
+        pending_status = CertificateStatus.PENDING
 
-    def list_certificates(self):
+        pending_certs = self.list_certificates(status=pending_status)
+        logger.debug(json.dumps(pending_certs))
+
+        for pc in pending_certs:
+            # Check also pc.get("transactionStatus"). Better safe than sorry.
+            if pc.get("transactionId") == certificate_id and pc.get("transactionStatus") == pending_status.value:
+                raise CertificateNotApprovedException
+
+        res = self.__make_post_request("/api/OrganizationValidatorSSL/GetSSLCertificate", data={"id": certificate_id})
+
+        if res.status_code == 404:
+            return None
+
+        return res.json()
+
+    def get_certificate_info(self, certificate_id):
+        cert_info = self.__make_post_request("/api/OrganizationValidatorSSL/GetSSLCertificate", data={"id": certificate_id})
+
+        return cert_info.json()
+
+    def get_user_certificate(self, certificate_id):
         """
-        Retrieves a list of valid certificates.
+        Retrieves a user certificate by its ID.
+
+        Args:
+            certificate_id (str): The certificate ID.
 
         Returns:
-            dict: List of certificates.
+            dict: The certificate details.
         """
-        json_payload = {"startIndex": 0, "status": "Valid", "filterPostDTOs": []}
-        data = self.__make_post_request("/api/OrganizationValidatorSSL/GetSSLTransactions", data=json_payload).json()
+        pending_status = CertificateStatus.PENDING
+
+        user_certs = self.list_user_certificates()
+        logger.debug(json.dumps(user_certs))
+
+        for uc in user_certs:
+            if uc.get("transactionId") == certificate_id and uc.get("status") == pending_status.value:
+                raise CertificateNotApprovedException
+
+        res = self.__make_post_request("/api/Certificate/GetCertificate", data={"id": certificate_id})
+
+        if res.status_code == 404:
+            return None
+
+        return res.json()
+
+    def list_certificates(
+        self, start_index: int = 0, status: CertificateStatus = CertificateStatus.VALID, full_info: bool = False
+    ):
+        """
+        Retrieves a list of certificates based on status and an optional email filter.
+
+        Args:
+            start_index (int): The starting index of the certificates to retrieve.
+            status (CertificateStatus): The status of the certificates to retrieve.
+
+        Returns:
+            list[dict]: List of certificates.
+        """
+
+        status_mapping = {
+            # On Harica GUI GetSSLTransactions are SSL Certificates
+            CertificateStatus.VALID: "GetSSLTransactions",
+            CertificateStatus.REVOKED: "GetSSLTransactions",
+            CertificateStatus.EXPIRED: "GetSSLTransactions",
+            # On Harica GUI GetSSLReviewableTransactions are SSL Requests
+            CertificateStatus.PENDING: "GetSSLReviewableTransactions",
+            CertificateStatus.READY: "GetSSLReviewableTransactions",
+            CertificateStatus.COMPLETED: "GetSSLReviewableTransactions",
+            CertificateStatus.CANCELLED: "GetSSLReviewableTransactions",
+        }
+
+        # Build the filters list only if an email is provided
+        # 19/03/2025 Filters are not working via API
+        # if a filter is specified API will ignore it and return all certificates
+        # This remains as example
+        # filters = [{
+        #     "filterType": "Email",
+        #     "filterTypeSelection": "Is",
+        #     "filterValue": email,
+        #     "isSeperator": False
+        # }] if email else []
+
+        json_payload = {"startIndex": start_index, "status": status.value, "filterPostDTOs": []}
+
+        endpoint = f"/api/OrganizationValidatorSSL/{status_mapping[status]}"
+        data = self.__make_post_request(endpoint, data=json_payload).json()
+
+        # Add status to each certificate
+        for cert in data:
+            cert["status"] = status.value
+
+            if full_info:
+                cert_info = self.get_certificate_info(cert["transactionId"])
+
+                for key, value in cert_info.items():
+                    cert[key] = value
+        return data
+
+    def list_user_certificates(self, full_info: bool = False):
+        """Retrieves a list of user certificates."""
+        endpoint = "/api/ServerCertificate/GetMyTransactions"
+        data = self.__make_post_request(endpoint).json()
+
+        # Add status to each certificate based on transactionStatus
+        # Set user because harica api will not return it
+        for cert in data:
+            # Fix harica mispelling of "transactionStatus" for cancelled status
+            if cert["transactionStatus"] == "Canceled":
+                cert["transactionStatus"] = "Cancelled"
+
+            # Revoked certificates have "Completed" as status and is not possible to
+            # retrieve only revoked user certs. Overwrite with "Revoked"
+            if cert["isRevoked"]:
+                cert["transactionStatus"] = CertificateStatus.REVOKED.value
+
+            cert["status"] = cert["transactionStatus"]
+            cert["user"] = self.full_name
+
+            if full_info:
+                cert_info = self.get_certificate_info(cert["transactionId"])
+
+                for key, value in cert_info.items():
+                    cert[key] = value
+
         return data
 
     def build_domains_list(self, domains):
         """
         Builds a list of domain information for the certificate request.
+        If a wildcard domain (*.domain.tld) exists alongside its base domain,
+        only the wildcard domain is processed since it already covers the base domain.
 
         Args:
             domains (list): List of domains.
@@ -189,13 +355,32 @@ class HaricaClient:
             list: List of domain dictionaries with validation details.
         """
         domains_info = []
+        processed_domains = set()
+        wildcard_bases = set()
+
+        # First, identify all wildcard domains and their base parts
         for dom in domains:
-            if dom in [f"www.{d}" for d in domains]:
+            if "*" in dom:
+                # Extract the base part of the wildcard domain (e.g., "mydomain.tld" from "*.mydomain.tld")
+                wildcard_base = dom.replace("*.", "")
+                wildcard_bases.add(wildcard_base)
+
+        for dom in domains:
+            # Check if this is a 'www.' domain and get the base domain
+            base_domain = dom.replace("www.", "")
+
+            # Skip if this domain has already been processed
+            if base_domain in processed_domains:
                 continue
+
+            # Skip regular domains that are covered by wildcards
+            if "*" not in dom and base_domain in wildcard_bases:
+                continue
+
             domain_info = {
                 "isWildcard": "*" in dom,
-                "domain": dom.replace("www.", ""),
-                "includeWWW": dom.startswith("www."),
+                "domain": base_domain,
+                "includeWWW": f"www.{base_domain}" in domains,
                 "isPrevalidated": True,
                 "isValid": True,
                 "isFreeDomain": True,
@@ -206,6 +391,7 @@ class HaricaClient:
                 "errorMessage": "",
                 "warningMessage": "",
             }
+            processed_domains.add(base_domain)
             domains_info.append(domain_info)
 
         return domains_info
@@ -224,7 +410,7 @@ class HaricaClient:
         data = self.__make_post_request("/api/ServerCertificate/CheckMachingOrganization", data=domains_info).json()
         return data
 
-    def request_certificate(self, domains, csr):
+    def request_certificate(self, domains, csr, transactionType="OV"):
         """
         Requests a new server certificate based on the provided domains and CSR.
 
@@ -241,7 +427,7 @@ class HaricaClient:
             raise ValueError("No available organization for this domain list")
 
         if len(organizations) > 1:
-            raise ValueError("Multiple organizations found. Specify one using '-O org'")
+            raise ValueError("Multiple organizations found.'")
 
         organization = organizations[0]
 
@@ -266,12 +452,14 @@ class HaricaClient:
             "domainsString": (None, json.dumps(domains_info)),
             "csr": (None, csr),
             "duration": (None, "1"),
-            "transactionType": (None, "OV"),
+            "transactionType": (None, transactionType),
             "friendlyName": (None, domains[0]),
             "isManualCSR": (None, "true"),
             "consentSameKey": (None, "true"),
-            "organizationDN": (None, orgDN),
         }
+        if transactionType == "OV":
+            payload["organizationDN"] = (None, orgDN)
+
         data = self.__make_post_request(
             "/api/ServerCertificate/RequestServerCertificate", data=payload, content_type="multipart/form-data"
         )
@@ -286,11 +474,16 @@ class HaricaClient:
             List: A list of pending transactions.
         """
         # Prepare the payload to retrieve SSL reviewable transactions
-        json_payload = {"startIndex": 0, "status": "Pending", "filterPostDTOs": []}
+        json_payload = {
+            "startIndex": 0,
+            "status": CertificateStatus.PENDING.value,
+            "filterPostDTOs": [],
+        }
 
         # Make a POST request to fetch the pending SSL transactions
         transactions = self.__make_post_request(
-            "/api/OrganizationValidatorSSL/GetSSLReviewableTransactions", data=json_payload
+            "/api/OrganizationValidatorSSL/GetSSLReviewableTransactions",
+            data=json_payload,
         ).json()
 
         return transactions
@@ -376,6 +569,55 @@ class HaricaClient:
         # Check if the cancellation was successful
         return response.status_code == 200
 
+    def revoke_certificate(self, cert_id: str) -> bool:
+        """
+        Revokes a certificate based on the provided certificate ID.
+
+        Args:
+            cert_id (str): The certificate ID to revoke.
+
+        Returns:
+            bool: True if the revocation was successful, False otherwise.
+
+        """
+        payload = {
+            "transactionId": cert_id,
+            # Name seems to be always 4.9.1.1.1.1
+            "name": "4.9.1.1.1.1",
+            "notes": f"Revoked via harica-cli by {self.email}",
+            "message": "",
+        }
+        response = self.__make_post_request(
+            "/api/OrganizationValidatorSSL/RevokeCertificate",
+            data=payload,
+        )
+
+        return response.status_code == 200
+
+    def revoke_user_certificate(self, cert_id: str) -> bool:
+        """
+        Revokes a user certificate based on the provided certificate ID.
+
+        Args:
+            cert_id (str): The certificate ID to revoke.
+
+        Returns:
+            bool: True if the revocation was successful, False otherwise.
+
+        """
+        payload = {
+            "transactionId": cert_id,
+            # Name seems to be always 4.9.1.1.1.1
+            "name": "4.9.1.1.1.1",
+            "notes": f"Revoked via harica-cli by {self.email}",
+        }
+        response = self.__make_post_request(
+            "/api/Certificate/RevokeCertificate",
+            data=payload,
+        )
+
+        return response.status_code == 200
+
     def list_domains(self):
         # Step 1: Search for the first available group in the organization
         groups = self.__make_post_request("/api/OrganizationAdmin/SearchGroups", data={"key": "", "value": ""}).json()
@@ -486,25 +728,33 @@ class HaricaClient:
         # Construct the full URL for the request
         url = f"{self.base_url}{endpoint}"
 
-        # Send the request based on the chosen method (GET or POST)
-        if method == "GET":
-            response = self.session.get(url)
-        elif method == "POST":
-            # Handle different content types for POST requests
-            if content_type == "multipart/form-data":
-                response = self.session.post(url, files=data)
+        try:
+            # Send the request based on the chosen method (GET or POST)
+            if method == "GET":
+                response = self.session.get(url)
+            elif method == "POST":
+                # Handle different content types for POST requests
+                if content_type == "multipart/form-data":
+                    response = self.session.post(url, files=data)
+                else:
+                    response = self.session.post(url, json=data)
             else:
-                response = self.session.post(url, json=data)
-        else:
-            raise ValueError("Unsupported method. Use 'GET' or 'POST'.")
+                raise ValueError("Unsupported method. Use 'GET' or 'POST'.")
 
-        if response.status_code == 400 or response.status_code == 401:
-            raise PermissionError(f"Permission Denied: {response.status_code}")
-        elif response.status_code != 200:
-            logging.error(f"API request failed with status code {response.status_code}: {response.text}")
+            if response.status_code == 400 or response.status_code == 401:
+                raise PermissionError("Permission denied.")
+            elif response.status_code != 200:
+                logging.error(f"API request failed with status code {response.status_code}: {response.text}")
 
-        # Return the response object for further processing
-        return response
+            # Return the response object for further processing
+            return response
+
+        except requests.exceptions.ProxyError as e:
+            logger.error(f"Proxy error during {method} request to {url}: {e}")
+            raise
+        except requests.exceptions.RequestException as e:
+            logger.error(f"Request error during {method} request to {url}: {e}")
+            raise
 
     def __make_get_request(self, endpoint, content_type="application/json"):
         """
@@ -532,3 +782,11 @@ class HaricaClient:
             Response: The response object from the POST request.
         """
         return self.__make_api_request(endpoint, method="POST", data=data, content_type=content_type)
+
+    def has_role(self, role: UserRole) -> bool:
+        """Check if the user has a specific role."""
+        return role in self.roles
+
+    def get_user_roles(self) -> str:
+        """Get a comma-separated string of user roles."""
+        return ", ".join(role.value for role in self.roles)
